-- name: [CS] Sackboy
-- description: The brand new Character Select mod of Sackboy, the stitched hero from LittleBigPlanet! \n\n\\#ff7777\\This Pack requires Character Select\nto use as a Library!

--[[
    API Documentation for Character Select can be found below:
    https://github.com/Squishy6094/character-select-coop/wiki/API-Documentation

    Use this if you're curious on how anything here works >v<
]]

local TEXT_MOD_NAME = "Sackboy"

-- Stops mod from loading if Character Select isn't on
if not _G.charSelectExists then
    djui_popup_create("\\#ffffdc\\\n"..TEXT_MOD_NAME.."\nRequires the Character Select Mod\nto use as a Library!\n\nPlease turn on the Character Select Mod\nand Restart the Room!", 6)
    return 0
end

local E_MODEL_SACKBOY = smlua_model_util_get_id("sackboy_geo") -- Located in "actors"

local E_MODEL_YELLOWHEAD = smlua_model_util_get_id("yellowhead_geo") -- Located in "actors"

local TEX_SACKBOY_LIFE_ICON = get_texture_info("sackboyicon") -- Located in "textures"

local TEX_YELLOWHEAD_LIFE_ICON = get_texture_info("yicon") -- Located in "textures"

-- All Located in "sound"
local VOICETABLE_SACK = {
    [CHAR_SOUND_ATTACKED] = 'SACKBOY_SOUND_ATTACKED.ogg',
    [CHAR_SOUND_DOH] = 'SACKBOY_SOUND_ATTACKED.ogg',
    [CHAR_SOUND_DROWNING] = 'SACKBOY_SOUND_DROWNING.ogg',
    [CHAR_SOUND_DYING] = 'SACKBOY_SOUND_DYING.ogg',
    [CHAR_SOUND_LETS_A_GO] = "SACKBOY_SOUND_LETS_A_GO.ogg",
    [CHAR_SOUND_HOOHOO] = 'SACKBOY_SOUND_YAHOO_WAHA_YIPPEE3.ogg',
    [CHAR_SOUND_ON_FIRE] = 'SACKBOY_SOUND_ON_FIRE.ogg',
    [CHAR_SOUND_OOOF] = 'SACKBOY_SOUND_OOOF.ogg',
    [CHAR_SOUND_OOOF2] = 'SACKBOY_SOUND_OOOF.ogg',
    [CHAR_SOUND_PUNCH_HOO] = 'SACKBOY_SOUND_YAHOO_WAHA_YIPPEE1.ogg',
    [CHAR_SOUND_PUNCH_WAH] = 'SACKBOY_SOUND_YAHOO_WAHA_YIPPEE3.ogg',
    [CHAR_SOUND_PUNCH_YAH] = 'SACKBOY_SOUND_YAHOO_WAHA_YIPPEE5.ogg',
    [CHAR_SOUND_SO_LONGA_BOWSER] = 'SACKBOY_SOUND_SO_LONGA_BOWSER.ogg',
    [CHAR_SOUND_TWIRL_BOUNCE] = 'SACKBOY_SOUND_YAHOO_WAHA_YIPPEE3.ogg',
    [CHAR_SOUND_WAAAOOOW] = 'SACKBOY_SOUND_WAAAOOOW.ogg',
    [CHAR_SOUND_WAH2] = 'SACKBOY_SOUND_YAHOO_WAHA_YIPPEE3.ogg',
    [CHAR_SOUND_YAHOO] = 'SACKBOY_SOUND_YAHOO_WAHA_YIPPEE1.ogg',
    [CHAR_SOUND_YAHOO_WAHA_YIPPEE] = { "SACKBOY_SOUND_YAHOO_WAHA_YIPPEE1.ogg", "SACKBOY_SOUND_YAHOO_WAHA_YIPPEE2.ogg", "SACKBOY_SOUND_YAHOO_WAHA_YIPPEE3.ogg", "SACKBOY_SOUND_YAHOO_WAHA_YIPPEE4.ogg", "SACKBOY_SOUND_YAHOO_WAHA_YIPPEE5.ogg"},
    [CHAR_SOUND_YAH_WAH_HOO] = { "SACKBOY_SOUND_YAHOO_WAHA_YIPPEE5.ogg", "SACKBOY_SOUND_YAHOO_WAHA_YIPPEE3.ogg", "SACKBOY_SOUND_YAHOO_WAHA_YIPPEE1.ogg"}
}

-- All Located in "actors"
local CAPTABLE_SACK = {
    normal = smlua_model_util_get_id("sackboys_cap_geo"),
    wing = smlua_model_util_get_id("sackboys_wing_cap_geo"),
    metal = smlua_model_util_get_id("sackboys_metal_cap_geo"),
    metalWing = smlua_model_util_get_id("sackboys_metal_wing_cap_geo"),
}

local PALETTE_SACK = {
    [PANTS]  = "9f7677",
    [SHIRT]  = "000000",
    [GLOVES] = "ffffff",
    [SHOES]  = "747474",
    [HAIR]   = "0000ff",
    [SKIN]   = "fec179",
    [CAP]    = "000000",
    [EMBLEM] = "0000ff",
}

local PALETTE_YELLOWHEAD = {
    [PANTS]  = "a018a0",
    [SHIRT]  = "000000",
    [GLOVES] = "ffffff",
    [SHOES]  = "747474",
    [HAIR]   = "000000",
    [SKIN]   = "fec179",
    [CAP]    = "e7e749",
    [EMBLEM] = "000000",
}

-- All Located in "textures"
local SackboyhealthMeter = {
    label = {
        left = get_texture_info("sackhealth2"),
        right = get_texture_info("sackhealth1"),
    },
    pie = {
        [1] = get_texture_info("sackboypowermeter8"),
        [2] = get_texture_info("sackboypowermeter7"),
        [3] = get_texture_info("sackboypowermeter6"),
        [4] = get_texture_info("sackboypowermeter5"),
        [5] = get_texture_info("sackboypowermeter4"),
        [6] = get_texture_info("sackboypowermeter3"),
        [7] = get_texture_info("sackboypowermeter2"),
        [8] = get_texture_info("sackboypowermeter1"),
    }
}

local CSloaded = false
local function on_character_select_load()
    CT_SACKBOY = _G.charSelect.character_add("Sackboy", "The main character of the LittleBigPlanet games. He is a small, creative, and heroic doll-like creature that lives in the planet Craftworld.", "Shiny", {r = 159, g = 118, b = 119}, E_MODEL_SACKBOY, CT_MARIO, TEX_SACKBOY_LIFE_ICON)
    _G.charSelect.character_add_caps(E_MODEL_SACKBOY, CAPTABLE_SACK)
    _G.charSelect.character_add_voice(E_MODEL_SACKBOY, VOICETABLE_SACK)
    _G.charSelect.character_add_palette_preset(E_MODEL_SACKBOY, PALETTE_SACK)
    _G.charSelect.character_add_health_meter(CT_SACKBOY, SackboyhealthMeter)
    --Yellow Head below
    _G.charSelect.character_add_costume(_G.charSelect.character_get_number_from_string("Sackboy"), "Yellow Head","The very first version of Sackboy!", "Shiny", {r = 231, g = 231, b = 73}, E_MODEL_YELLOWHEAD, CT_MARIO, TEX_YELLOWHEAD_LIFE_ICON)
    _G.charSelect.character_edit_costume(_G.charSelect.character_add_palette_preset(E_MODEL_YELLOWHEAD, PALETTE_YELLOWHEAD))
    _G.charSelect.character_edit_costume( _G.charSelect.character_add_voice(E_MODEL_YELLOWHEAD, VOICETABLE_SACK))
    _G.charSelect.character_edit_costume(_G.charSelect.character_add_caps(E_MODEL_YELLOWHEAD, CAPTABLE_SACK))
    CSloaded = true
end

local function on_character_sound(m, sound)
    if not CSloaded then return end
    if _G.charSelect.character_get_voice(m) == VOICETABLE_SACK then return _G.charSelect.voice.sound(m, sound) end
end

local function on_character_snore(m)
    if not CSloaded then return end
    if _G.charSelect.character_get_voice(m) == VOICETABLE_SACK then return _G.charSelect.voice.snore(m) end
end

hook_event(HOOK_ON_MODS_LOADED, on_character_select_load)
hook_event(HOOK_CHARACTER_SOUND, on_character_sound)
hook_event(HOOK_MARIO_UPDATE, on_character_snore)